(*
  This projects demonstrates the TCP Sniffing capability of TSniffer. Note that
  The display will only show TCP packets however it is possible (in non filtered
  mode for the application to capture all packets.

  You can toggle the filtering option by clicking on the filter status portion
  of the status bar.

  In filter mode the underlaying packet capture engine will only capture the
  packets that fullfill the filter criteria "TCP" in this case. Also the caplen
  is effective in this case, meaning that only a portion of the packet will be
  captured.
*)

unit TCPSniffer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ToolWin, ComCtrls, ExtCtrls, Sniffer,winpcaptypes,winsock,
  Buttons;

type
  TForm1 = class(TForm)
    Sniffer1: TSniffer;
    Panel1: TPanel;
    Panel2: TPanel;
    ListView1: TListView;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    StatusBar1: TStatusBar;
    ComboBox1: TComboBox;
    Label1: TLabel;
    procedure BitBtn1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure Sniffer1Capture(const Header: PTpcap_pkthdr;
      const Data: Pointer);
    procedure Sniffer1CaptureTCP(const Header: PTpcap_pkthdr;
      const Data: PTCPPckt);
    procedure ComboBox1Change(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  cap  : Integer=0;
  disp : Integer=0;
  MaxCapLen : Integer=0;
  MaxLen : Integer=0;
implementation

{$R *.dfm}

procedure TForm1.BitBtn1Click(Sender: TObject);
Var ErrStr : String;

begin
//  Sniffer1.Filtered:=True;
  If Sniffer1.Activate=0 then
  Begin
//    StatusBar1.Panels[8].Text:=IntToStr(Sniffer1.CaptureLength);
    ComboBox1.Enabled:=False;
    StatusBar1.Panels[1].Text:='Active';
    If Sniffer1.Filtered then
      Statusbar1.Panels[3].Text:='Yes'
    Else
      Statusbar1.Panels[3].Text:='No';
  End
  Else
  Statusbar1.Panels[8].Text:=' '+ErrStr;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Label1.Parent:=StatusBar1;
  If Sniffer1.Filtered Then
    Label1.Caption:='YES';

  ComboBox1.Items:=Sniffer1.DeviceDescriptions;
  If ComboBox1.Items.Count>0 then
  Begin
    Sniffer1.DeviceIndex:=0;
    ComboBox1.Text:=Sniffer1.DeviceDescription;
  End;
end;

procedure TForm1.BitBtn3Click(Sender: TObject);
begin
  ListView1.Clear;
end;

procedure TForm1.BitBtn2Click(Sender: TObject);
Var ErrStr : String;
begin
  MaxCapLen:=0;
  MaxLen:=0;
  If Sniffer1.Deactivate=0 then
  Begin
    StatusBar1.Panels[1].Text:='In-active';
    StatusBar1.Panels[3].Text:='No';
    ComboBox1.Enabled:=True;
  End
  Else
    Statusbar1.Panels[8].Text:=' '+ErrStr;
end;

procedure TForm1.Sniffer1Capture(const Header: PTpcap_pkthdr;
  const Data: Pointer);
begin
  Inc(Cap);
  Statusbar1.Panels[5].Text:=IntToStr(Cap);
end;

procedure TForm1.Sniffer1CaptureTCP(const Header: PTpcap_pkthdr;
  const Data: PTCPPckt);
begin
With Data.TCPHeader do
  Begin
    With ListView1.Items.Add do
    Begin
      Caption:=IntToStr(IPHeader.Service);
      SubItems.Add(IntToStr(htons(IPHeader.Length)));
      SubItems.Add(IntToStr(htons(IPHeader.Ident)));
      SubItems.Add(IntToStr(IPHeader.TimeLive));
      SubItems.Add(IntToStr(IPHeader.Protocol));
      SubItems.Add(IntToStr(ntohs(IPHeader.Checksum)));
      SubItems.Add(IPToStr(ntohl(IPHeader.Source)));
      SubItems.Add(IntToStr(ntohs(Source)));
      SubItems.Add(IPToStr(ntohl(IPHeader.Destination)));
      SubItems.Add(IntToStr(ntohs(Destination)));
    End;
    Inc(disp);
    Statusbar1.Panels[7].Text:=IntToStr(disp);
    If Header.CapLen>MaxCapLen Then
      MaxCapLen:=Header.CapLen;

    If Header.Len>MaxLen Then
      MaxLen:=Header.Len;

    Statusbar1.Panels[9].Text:=IntToStr(Header.CapLen)+'/'+IntToStr(MaxCapLen);
    Statusbar1.Panels[11].Text:=IntToStr(Header.Len)+'/'+IntToStr(MaxLen);
    End;
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  Sniffer1.DeviceIndex:=ComboBox1.ItemIndex;
end;

procedure TForm1.Label1Click(Sender: TObject);
begin
  If not ComboBox1.Enabled then Exit;
  Sniffer1.Filtered:= Not Sniffer1.Filtered;
  If Sniffer1.Filtered then
    Label1.Caption:='YES'
  Else
    Label1.Caption:='NO';
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  BitBtn2Click(self);
end;

end.
